# 웹 프로그래밍 - 4. 웹 앱 개발:예약서비스 2/4

## 2. 라이브러리 활용과 클린코드 - FE

### handlebar를 이용하여 템플릿 작업하기

[handlebar 다운로드](https://cdnjs.com/libraries/handlebars.js/)

[실습 전체코드](./handlebarTest.html)

#### hadlebar 실습 #1

- HTML파일 내부에 `<script>`태그로 저장해 둔 템플릿에 데이터를 추가하여 화면에 나타내기

- replace를 사용하지 않고 데이터를 치환 가능

```html
<script type="myTemplate" id="listTemplate">
    <li>
        <div>게시자 : {{name}}</div>
        <div class="content">{{content}}</div>
        <div>좋아요 개수 : <span> {{like}} </span></div>
        <div>
        	<div>{{comment}}</div>
    	</div>
    </li>
</script>
```

```javascript
var data = {
    "id" : 1,
    "name" : "홍길동",
    "content" : "새로운 글!",
    "like" : 5,
    "comment" : "반가워요~!"
};

var template = document.querySelector("#listTemplate").innerText;

// pre-compile 값이 들어오면 바로 작업을 할 수 있게 변수를 기억해 둠
var bindTemplate = Handlebars.compile(template);  // bindTemplate은 메서드
var resultHTML = bindTemplate(data);

var show = document.querySelector("#show");
show.innerHTML = resultHTML;
```



#### handlebar 실습 #2 배열이 포함된 데이터 처리하기

```HTML
<script type="myTemplate" id="listTemplate">
    <li>
        <div>게시자 : {{name}}</div>
        <div class="content">{{content}}</div>
        <div>좋아요 개수 : <span> {{like}} </span></div>
        <h3>댓글목록</h3>
        {{#each comment}}
			<div>{{@index}}번째 댓글 : {{this}}</div>
        {{/each}}
    </li>
</script>
```

```javascript
var data = {
    "id" : 1,
    "name" : "홍길동",
    "content" : "새로운 글!",
    "like" : 5,
    "comment" : ["반가워요~!", "잘 보고 갑니다"]
};
```



#### handlebar 실습 #3 여러 개의 데이터 처리하기

```javascript
var data = [
        {"id" : 1, "name" : "홍길동", "content" : "새로운 글!", "like" : 5, "comment" : ["반가워요~!", "잘 보고 갑니다"]},
        {"id" : 2, "name" : "김철수", "content" : "두 번째 글!", "like" : 2, "comment" : []},
        {"id" : 3, "name" : "김영희", "content" : "세 번째 글!", "like" : 0, "comment" : ["안녕하세요!", "잘 보고 갑니다~"]}
];
```

내 답안

```javascript
var resultHTML = "";

var show = document.querySelector("#show");

data.forEach(function(v) {
    resultHTML += bindTemplate(v);
});

show.innerHTML = resultHTML;
```

강의 실습

- `reduce()`는 누적된 합을 구하는데 용이

```javascript
// ""을 첫 번째 값으로 지정
// next를 이용해 나온 값을 prev에 합치기
// 재귀처럼 동작
var resultHTML = data.reduce(function(prev, next) {
    return prev + bindTemplate(next);
}, "");

show.innerHTML = resultHTML;
```



#### handlebar 실습 #4 조건 상황에 따른 처리

댓글이 하나도 없다면 메세지 띄우기

```HTML
<script type="myTemplate" id="listTemplate">
    <li>
        <div>게시자 : {{name}}</div>
        <div class="content">{{content}}</div>
        <div>좋아요 개수 : <span> {{like}} </span></div>
        <h4>댓글목록</h4>
        {{#if comment}}
        	{{#each comment}}
            	<div>{{@index}}번째 댓글 : {{this}}</div>
        	{{/each}}
        {{else}}
        	<div>첫 번째 댓글을 달아주세요!</div>
        {{/if}}
    </li>
</script>
```



#### handlebar 실습 #5 help function 사용하기

좋아요가 5개 이상이면 특정 문구를  출력하기

```html
<script type="myTemplate" id="listTemplate">
	<li>
      	<div>게시자 : {{name}}</div>
      	<div class="content">{{content}}</div>

      	{{#likes like}}
        	{{like}}
      	{{/likes}}

      	<h4>댓글목록</h4>
      	{{#if comment}}
      		{{#each comment}}
        		<div>{{@index}}번째 댓글 : {{this}}</div>
        	{{/each}}
    	{{else}}
      		<div>첫 번째 댓글을 달아주세요!</div>
    	{{/if}}
    </li>
</script>
```

```javascript
Handlebars.registerHelper("likes", function(like) {
    if(like > 4) {
        return "<span>좋아요가 " + like + "개 이상이네요! 와우~</span>";
    } else if(like < 1) {
        return "아직 좋아요가 없어요ㅠㅠ";
    } else {
        return like;
    }
});
```



#### 참고

- [handlebars built-in helpers](https://handlebarsjs.com/guide/builtin-helpers.html)
- [Tagged template literal](https://developer.mozilla.org/ko/docs/Web/JavaScript/Reference/Template_literals)



### 클린코드

클린코드는 읽기 좋은, 가독성 높은 코드

#### 1. 이름(Naming Convention)

- 함수의 목적에 맞는 이름인가?
- 함수의 이름이 동사 + 명사 인가?
- 함수가 함수의 이름에 맞는 하나의 로직을 담고 있는가?
- 변수는 명사이며 의미있는 이름인가?
- 표기법 확인

#### 2. 의미가 드러난 구현 패턴

```javascript
let radius = 5;
let area = radius * radius * 3.14;

// 더 좋은 방법
const PI = 3.14;
let radius = 5;
let area = radius * radius * PI;
```

- CSS를 쓸 때도 3번째 자식에 적용하는 등의 방식은 좋지 않다. 3번 째 자식이 무엇인지 바로 인지할 수 없기 때문이다.

#### 3. 가능한 Scope를 좁게

- 불필요한 전역변수를 최소화 한다

#### 4. 빨리 반환하여 if문 중첩없애기

```javascript
function test(lastName, age) {
    if(age > 25) {
        if(lastName === "Kim") {
            return true;
        }
    }
}

// 더 좋은 방법
function test(lastName, age) {
    if(age <= 25) return;
    
    if(lastName === "Kim") {
        return true;
    }
}
```



#### 참고

- [Airbnb JavaScript Style Guide(국문)](https://github.com/tipjs/javascript-style-guide)
- [ESLint](https://eslint.org/)

<br>

---

[HOME](https://github.com/tunaep5/Boostcourse/blob/master/README.md)
